Object subclass: #GSPTreeNodeBuilder
	instanceVariableNames: 'nodeTypes '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector Main'!
GSPTreeNodeBuilder comment:
'My instances build nodes for the reference tree used to inspect objects. Given an object, they choose the appropriate node class, and they make a new node for this object.

Instance Variables:
	nodeTypes	<(SequenceableCollection of: (Association | VariableBinding))>	description of nodeTypes

'!


!GSPTreeNodeBuilder methodsFor: 'private'!

defaultNodesType
	" The receiver makes and returns a collection (for default) of blocks that make nodes of reference tree."

	| col |
	self nodeTypes isNil
		ifTrue: 
			[col := OrderedCollection new.
			col
				add: Behavior -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPBehaviorTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: UndefinedObject -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPNullTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: Magnitude -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPMagnitudeTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: Context -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPContextTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: CompiledCode -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPCompiledCodeTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: String -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPStringTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: KeyedCollection -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPKeyedCollectionTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: Dictionary -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPKeyedCollectionTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: SequenceableCollection -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPSequenceableCollectionTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: Collection -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPCollectionTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: BlockClosure -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPBlockClosureTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent].
			col
				add: Object -> 
							[:nameAndObject :indexAndReference :parent | 
							GSPDefaultTreeNode newWithObject: nameAndObject value
								name: nameAndObject key
								index: indexAndReference key
								parentNode: parent]]
		ifFalse: [col := self nodeTypes].
	^col!

nodeTypes
	" The receiver returns a collection of blocks that make nodes of reference tree."

	^nodeTypes!

nodeTypes: anObject 
	" The receiver stores a collection of blocks that make nodes of reference tree."

	nodeTypes := anObject! !

!GSPTreeNodeBuilder methodsFor: 'accessing'!

createNodeForObject: anObject name: aName index: anIndex parentNode: aParent 
	" The receiver makes a tree node, according to the data received by parameter."

	| block |
	block := anObject isBehavior 
				ifTrue: [self nodeTypes first value]
				ifFalse: 
					[(self nodeTypes 
						detect: [:each | anObject class includesBehavior: each key]) value].
	^block 
		value: aName -> anObject
		value: anIndex -> aParent value
		value: aParent! !

!GSPTreeNodeBuilder methodsFor: 'initialize-release'!

initialize
	" The receiver is initialized."

	self nodeTypes: self defaultNodesType! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPTreeNodeBuilder class
	instanceVariableNames: ''!


!GSPTreeNodeBuilder class methodsFor: 'instance-creation'!

createNodeForObject: anObject 
	^self new 
		createNodeForObject: anObject
		name: nil
		index: 0
		parentNode: nil!

createNodeForObject: anObject name: aName index: anIndex parentNode: aParent 
	^self new 
		createNodeForObject: anObject
		name: aName
		index: anIndex
		parentNode: aParent!

new
	^super new initialize! !

ApplicationModel subclass: #GSPInspector
	instanceVariableNames: 'structure selectionValue oldSelection treeVisitor inspectedObject '
	classVariableNames: 'SelectionMenu TreeViewMenu '
	poolDictionaries: ''
	category: 'GSP- Inspector Main'!
GSPInspector comment:
'My instances inspect objects in a way more "friendly" that the traditional inspector.

Instance Variables:
	inspectedObject	<Object>	description of inspectedObject
	oldSelection	<Object>	description of oldSelection
	selectionValue	<ValueModel>	description of selectionValue
	structure	<SelectionInList>	description of structure
	treeVisitor	<TreeNodeVisitor>	description of treeVisitor

Class Variables:
	SelectionMenu	<>	description of SelectionMenu
	TreeViewMenu	<>	description of TreeViewMenu

'!


!GSPInspector methodsFor: 'selection-private'!

disableAllSelectionMenuItems
	" The receiver disables all the items of the menu of the text field. "

	1 to: SelectionMenu numberOfItems
		do: [:i | (SelectionMenu menuItemAt: i) disable]!

enableSelectionMenuItems
	"The receiver enables all the items of the menu of the text field if the selected object can be modified."

	self selectedNode canChange 
		ifTrue: 
			[self unhideAndHideSelectionMenuItems.
			1 to: SelectionMenu numberOfItems
				do: [:i | (SelectionMenu menuItemAt: i) enable]]
		ifFalse: [self disableAllSelectionMenuItems]!

oldSelection
	" The receiver returns the string of the text field before being accepted."

	^oldSelection!

oldSelection: anObject 
	" The receiver stores the string of the text field before being accepted."

	oldSelection := anObject!

refreshSelection
	"The receiver upgrades the selected node so that it reflects the changes of its value."

	| parentIndex currentIndex |
	currentIndex := self selectedIndexNode.
	parentIndex := self treeModel parentIndexOfChildAt: self selectedIndexNode.
	self selectedIndexNode: parentIndex.
	self contractTree.
	self selectedIndexNode: parentIndex.
	self selectedNode initializeChildren.
	self expandTree.
	self selectedIndexNode: parentIndex.
	self expandTree!

showSelectionMenuItems
	" If a selected node exists, it enables the items of the menu of the text field, in case  
	contrary it disables them."

	self selectedNode isNil 
		ifTrue: [self disableAllSelectionMenuItems]
		ifFalse: [self enableSelectionMenuItems]!

showSelectionMenuItemsForCompiledCode
	" The receiver shows the items of the menu of the text field that allow to compile code."

	SelectionMenu hideItem: (SelectionMenu menuItemAt: 1).
	SelectionMenu hideItem: (SelectionMenu menuItemAt: 2).
	SelectionMenu unhideItem: (SelectionMenu menuItemAt: 3)!

showSelectionMenuItemsForDefault
	" The receiver shows the items of the menu of the text field that allow to evaluate the text inside it."

	SelectionMenu unhideItem: (SelectionMenu menuItemAt: 1).
	SelectionMenu unhideItem: (SelectionMenu menuItemAt: 2).
	SelectionMenu hideItem: (SelectionMenu menuItemAt: 3)!

unhideAndHideSelectionMenuItems
	" If the selected node represents a compiled code, the receiver shows the items of the menu of the text field that allow to compile code, in case it represents another object, it shows the items that allow to evaluate text or to accept it as such."

	self selectedNode canCompile 
		ifTrue: [self showSelectionMenuItemsForCompiledCode]
		ifFalse: [self showSelectionMenuItemsForDefault]! !

!GSPInspector methodsFor: 'initialize-release'!

initialize
	"The receiver initializes the variables that can be initialized before opening the window. "

	structure := SelectionInList new.
	^self! !

!GSPInspector methodsFor: 'accessing'!

inspect: anObject 
	" The receiver receives the object to be inspected. "

	self inspectedObject: anObject.
	self initialize!

inspectedObject
	" The receiver returns the object to be inspected. "

	^inspectedObject!

inspectedObject: anObject 
	" The receiver stores the object to be inspected. "

	inspectedObject := anObject! !

!GSPInspector methodsFor: 'private'!

setWindowLabel
	" The receiver sets the label of the window with the identification from the object to be inspected. "

	self builder window 
		newLabel: ('Graphic Inspector in ' , self inspectedObject printString) 
				asText! !

!GSPInspector methodsFor: 'interface opening'!

postOpenWith: aBuilder 
	" This method is executed after being created the application, but before  
open up the window, and it initializes the components that can be obtained by means of the builder.  "

	self showInspectedObject!

showInspectedObject
	self setWindowLabel.
	self drawTree.
	self disableAllTreeMenuItems.
	self disableAllSelectionMenuItems! !

!GSPInspector methodsFor: 'menu-creation'!

createSelectionMenu
	" The receiver returns the menu for the text field.
      GInspector flushMenus"

	SelectionMenu == nil 
		ifTrue: 
			[SelectionMenu := Menu 
						labels: 'accept evaluation\accept as text\compile\cancel' withCRs
						lines: #(3)
						values: #(#acceptSelection #acceptSelectionAsText #acceptSelection #cancelSelection)].
	^SelectionMenu!

createTreeMenu
	" The receiver returns the menu for the tree of nodes.
      GInspector flushMenus"

	TreeViewMenu == nil 
		ifTrue: 
			[TreeViewMenu := Menu 
						labels: 'contract\expand\refresh all\inspect\basic inspect\gInspect\browse\owners\reference path' 
								withCRs
						lines: #(3 6)
						values: #(#contractTree #expandTree #refreshAll #inspectField #basicInspectField #gInspectField #browseField #ownersOfField #refPathToField)].
	^TreeViewMenu!

selectionMenuHolder
	" The receiver makes and returns a menu for the text field."

	^[self createSelectionMenu]!

treeMenuHolder
	" The receiver makes and returns a menu for the tree of nodes."

	^[self createTreeMenu]! !

!GSPInspector methodsFor: 'aspects'!

selectionValue
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined.  The
	initialization provided below may have been preempted by an
	initialize method."

	^selectionValue isNil
		ifTrue:
			[selectionValue := String new asValue]
		ifFalse:
			[selectionValue]!

structure
	"This method was generated by UIDefiner.  Any edits made here
	may be lost whenever methods are automatically defined."

	^structure! !

!GSPInspector methodsFor: 'accessing-private'!

nodeAt: anIndex 
	" The receiver returns the node that has index anIndex in the tree of nodes."

	^self structure list at: anIndex!

selectedIndexNode
	" The receiver returns the index of the selected node."

	^self structure selectionIndex!

selectedIndexNode: anIndex 
	" The receiver stores the index of the selected node."

	self structure selectionIndex: anIndex!

selectedNode
	" The receiver returns the selected node."

	^self structure selection!

selectedNode: aNode 
	" The receiver stores the selected node."

	self structure selection: aNode!

treeModel
	" The receiver returns the model of the tree."

	^self structure list!

treeVisitor
	" The receiver returns the object that collaborates in the armed of the tree of references for the inspected object."

	^treeVisitor!

treeVisitor: anObject 
	" The receiver stores the object that collaborates in the armed of the tree of references for the inspected object."

	treeVisitor := anObject!

treeVisitorClass
	" The receiver returns the appropriate class for the object that collaborates in the armed of the tree of references for the inspected object."

	^GSPTreeNodeVisitor! !

!GSPInspector methodsFor: 'tree-private'!

contractTree
	"Contrae el nivel del arbol de referencias donde se encuentra seleccionado algun nodo."

	self selectedIndexNode = 0 
		ifFalse: 
			[self treeModel contractFully: self selectedIndexNode.
			self invalidateSelectedNode.
			self showSelection]!

disableAllTreeMenuItems
	"Deshabilita todos los items del menu del arbol de referencias."

	1 to: TreeViewMenu numberOfItems
		do: [:i | (TreeViewMenu menuItemAt: i) disable]!

drawTree
	"Se evalua cuando se abre el inspector para mostrar el objeto inspeccionado junto con sus 
	referencias directas."

	| tree |
	self treeVisitor: self treeVisitorClass new.
	tree := self treeVisitor createTreeNode: self inspectedObject.
	self treeVisitor referencesFromNode: tree.
	self treeModel root: tree.
	self treeModel expand: 1!

enableTreeMenuItems
	" Habilita todos los items del menu del arbol de referencias. "

	1 to: TreeViewMenu numberOfItems
		do: [:i | (TreeViewMenu menuItemAt: i) enable]!

expandTree
	" Expande el nodo seleccionado.  "

	self selectedNode isNil 
		ifFalse: [self selectedNode expanded ifFalse: [self growTree]].
	self selectedIndexNode = 0 
		ifFalse: 
			[(self treeModel wrapperAt: self selectedIndexNode) children: nil.
			self treeModel expand: self selectedIndexNode]!

growTree
	" Expande el nodo seleccionado de manera que encuentre todos los elementos que referencia. "

	self treeVisitor referencesFromNode: self selectedNode!

invalidateSelectedNode
	self selectedIndexNode: 0.
	self selectedNode: nil!

showTreeMenuItems
	"Si existe un objeto seleccionado habilita los items del menu del arbol de referencias, en caso
	contrario los deshabilita. "

	self selectedNode isNil 
		ifTrue: [self disableAllTreeMenuItems]
		ifFalse: [self enableTreeMenuItems]! !

!GSPInspector methodsFor: 'actions'!

showSelection
	" Muestra las caracteristicas del objeto seleccionado en el campo de texto. "

	self selectedNode isNil 
		ifFalse: 
			[self selectionValue value: self selectedNode displayValue.
			self oldSelection: self selectionValue value].
	self showTreeMenuItems.
	self showSelectionMenuItems! !

!GSPInspector methodsFor: 'selection-menu'!

acceptSelection
	"Acepta el codigo del campo de texto como nuevo valor del objeto seleccionado. En caso de que 
	el nuevo valor sea incorrecto, recupera el codigo del valor anterior ."

	
	[self treeVisitor acceptNewValue: self selectionValue value
		forNode: self selectedNode.
	self refreshSelection] 
			on: Error
			do: [:exc | ^self cancelSelection]!

acceptSelectionAsText
	"Acepta el codigo del campo de texto como nuevo valor del objeto seleccionado. En caso de que 
	el nuevo valor sea incorrecto, recupera el codigo del valor anterior ."

	
	[self treeVisitor acceptStringAsValue: self selectionValue value
		forNode: self selectedNode.
	self refreshSelection] 
			on: Error
			do: [:exc | ^self cancelSelection]!

cancelSelection
	"Cancela la modificacion del valor del objeto seleccionado. Restablece el codigo del valor que tenia 
	el objeto cuando fue inicialmente seleccionado."

	self selectionValue value: self oldSelection.
	self showSelection!

refreshAll
	self selectedNode: (self nodeAt: 1).
	self selectedIndexNode: 1.
	self selectedNode initializeChildren.
	self contractTree.
	self selectedIndexNode: 1.
	self expandTree! !

!GSPInspector methodsFor: 'tree-menu'!

basicInspectField
	" Efectua una inspeccion basica sobre el objeto seleccionado."

	self selectedNode value basicInspect!

browseField
	"Browsea el objeto seleccionado."

	self selectedNode value browse!

gInspectField
	"Realiza una inspeccion con el inspector grafico del objeto seleccionado."

	self selectedNode value gInspect!

inspectField
	"Efectua una inspeccion sobre el objeto seleccionado. "

	self selectedNode value inspect!

ownersOfField
	| wa |
	wa := self selectedNode value allOwnersWeakly: true.
	wa 
		indexOf: self
		replaceWith: 0
		startingAt: 1
		stoppingAt: wa size.
	wa inspect!

refPathToField
	| rpcc rpc |
	rpcc := Smalltalk at: #ReferencePathCollector ifAbsent: [^self].
	rpc := rpcc new.
	rpc ignore: self; ignore: thisContext.
	Cursor execute showWhile: [(rpc someReferencePathTo: self selectedNode value) inspect]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPInspector class
	instanceVariableNames: ''!


!GSPInspector class methodsFor: 'interface specs'!

windowSpec
	"UIPainter new openOnClass: self andSelector: #windowSpec"

	<resource: #canvas>
	^#(#FullSpec 
		#window: 
		#(#WindowSpec 
			#label: 'Graphic Inspector' 
			#bounds: #(#Rectangle 145 119 668 577 ) 
			#flags: 8 
			#isEventDriven: true ) 
		#component: 
		#(#SpecCollection 
			#collection: #(
				#(#ICC2TreeViewSpec 
					#layout: #(#LayoutFrame 0 0.00773694 0 0.00884956 0 0.990329 0 0.63938 ) 
					#name: #structure 
					#flags: 15 
					#isOpaque: true 
					#model: #structure 
					#callbacksSpec: 
					#(#UIEventCallbackSubSpec 
						#focusInSelector: #expandTree 
						#valueChangeSelector: #showSelection 
						#doubleClickSelector: #expandTree ) 
					#menu: #treeMenuHolder 
					#useModifierKeys: true 
					#selectionType: #highlight 
					#useIcons: #other 
					#iconSelectors: #(#valueImage ) 
					#autoSelect: true ) 
				#(#TextEditorSpec 
					#layout: #(#LayoutFrame 0 0.00773694 0 0.652655 0 0.990329 0 0.878319 ) 
					#name: #selectionValue 
					#model: #selectionValue 
					#menu: #selectionMenuHolder ) 
				#(#CompositeSpecCollection 
					#collection: #(
						#(#ArbitraryComponentSpec 
							#layout: #(#LayoutFrame 0 0.0216535 0 0.519231 0 0.0748031 0 1.0 ) 
							#name: #magnitude 
							#flags: 0 
							#component: #magnitude ) 
						#(#ArbitraryComponentSpec 
							#layout: #(#LayoutFrame 0 0.454724 0 0.519231 0 0.507874 0 1.0 ) 
							#name: #collection 
							#flags: 0 
							#component: #collection ) 
						#(#ArbitraryComponentSpec 
							#layout: #(#LayoutFrame 0 0.830709 0 0.519231 0 0.88189 0 1.0 ) 
							#name: #null 
							#flags: 0 
							#component: #null ) 
						#(#GroupBoxSpec 
							#layout: #(#LayoutFrame 0 0.0 0 0.0 0 1.0 0 0.942308 ) ) 
						#(#ArbitraryComponentSpec 
							#layout: #(#LayoutFrame 0 0.0216535 0 0.115385 0 0.0748031 0 0.596154 ) 
							#name: #behavior 
							#flags: 0 
							#component: #behavior ) 
						#(#ArbitraryComponentSpec 
							#layout: #(#LayoutFrame 0 0.454724 0 0.115385 0 0.507874 0 0.596154 ) 
							#name: #string 
							#flags: 0 
							#component: #string ) 
						#(#ArbitraryComponentSpec 
							#layout: #(#LayoutFrame 0 0.830709 0 0.115385 0 0.88189 0 0.596154 ) 
							#name: #default 
							#flags: 0 
							#component: #default ) 
						#(#LabelSpec 
							#layout: #(#LayoutOrigin 0 0.866142 0 0.0 ) 
							#label: 'Other' 
							#style: #default ) 
						#(#LabelSpec 
							#layout: #(#LayoutOrigin 0 0.0570866 0 0.0 ) 
							#label: 'Behavior subclass' 
							#style: #default ) 
						#(#LabelSpec 
							#layout: #(#LayoutOrigin 0 0.492126 0 0.0 ) 
							#label: 'A string' 
							#style: #default ) 
						#(#LabelSpec 
							#layout: #(#LayoutOrigin 0 0.492126 0 0.384615 ) 
							#label: 'A collection' 
							#style: #default ) 
						#(#LabelSpec 
							#layout: #(#LayoutOrigin 0 0.0570866 0 0.384615 ) 
							#label: 'A magnitude ' 
							#style: #default ) 
						#(#LabelSpec 
							#layout: #(#LayoutOrigin 0 0.866142 0 0.384615 ) 
							#label: 'Nil' 
							#style: #default ) ) 
					#compositeSpec: 
					#(#CompositeSpec 
						#layout: #(#LayoutFrame 0 0.00967118 0 0.887168 0 0.992263 0 1.00221 ) ) ) ) ) )! !

!GSPInspector class methodsFor: 'view creation'!

open: anAGInspector 
	"Create and schedule a window on the model, anInspector."

	anAGInspector open!

openOn: anObject 
	| gi |
	gi := self inspect: anObject.
	^gi open! !

!GSPInspector class methodsFor: 'instance creation'!

inspect: anObject 
	"Answer an instance of me to provide an inspector for anObject."
	^self new inspect: anObject! !

!GSPInspector class methodsFor: 'class initialization'!

flushMenus
	TreeViewMenu := SelectionMenu := nil! !

!GSPInspector class methodsFor: 'resources'!

behavior
	"UIMaskEditor new openOnClass: self andSelector: #behavior"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue magenta; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 4227 scaledGreen: 0 scaledBlue: 4227); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 0 5 88 37 42 129 88 37 41 97 88 37 42 133 88 37 41 97 88 37 41 97 88 37 42 129 88 37 0 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])!

collection
	"UIMaskEditor new openOnClass: self andSelector: #collection"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue green; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 0 scaledGreen: 4227 scaledBlue: 0); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 64 5 88 37 10 129 88 37 41 97 88 37 41 85 88 37 41 85 88 37 41 97 88 37 10 129 88 37 64 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])!

default
	"UIMaskEditor new openOnClass: self andSelector: #default"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue red; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 4227 scaledGreen: 0 scaledBlue: 0); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 64 5 88 37 10 129 88 37 41 97 88 37 41 97 88 37 41 97 88 37 41 97 88 37 10 129 88 37 64 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])!

magnitude
	"UIMaskEditor new openOnClass: self andSelector: #magnitude"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue yellow; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 4227 scaledGreen: 4227 scaledBlue: 0); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 1 65 88 37 33 33 88 37 40 161 88 37 34 33 88 37 32 33 88 37 33 33 88 37 33 33 88 37 1 1 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])!

null
	"UIMaskEditor new openOnClass: self andSelector: #null"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue cyan; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 0 scaledGreen: 4227 scaledBlue: 4227); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 0 65 88 37 40 33 88 37 42 33 88 37 40 161 88 37 40 97 88 37 40 97 88 37 40 97 88 37 0 65 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])!

string
	"UIMaskEditor new openOnClass: self andSelector: #string"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue blue; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 0 scaledGreen: 0 scaledBlue: 4227); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 64 5 88 37 10 161 88 37 37 85 88 37 10 133 88 37 80 33 88 37 37 97 88 37 10 129 88 37 64 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

Object variableSubclass: #GSPTreeNodeVisitor
	instanceVariableNames: 'treeNodeBuilder '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector Main'!
GSPTreeNodeVisitor comment:
'My instances collaborate in the construction of the reference tree used to inspect objects. They act as mediators in this construction process.

Instance Variables:
	treeNodeBuilder	<TreeNodeBuilder | TreeNodeBuilder class>	description of treeNodeBuilder

'!


!GSPTreeNodeVisitor methodsFor: 'accessing'!

acceptNewValue: aNewValue forNode: aNode 
	"The receiver modifies aNode with the new value aNewValue."

	aNode acceptNewValue: aNewValue with: self treeNodeBuilder!

acceptStringAsValue: aString forNode: aNode 
	" The receiver modifies aNode with the new string aString."

	aNode acceptStringAsValue: aString with: self treeNodeBuilder!

createTreeNode: anObject 
	"The receiver makes a new node for anObject."

	^self treeNodeBuilder
		createNodeForObject: anObject
		name: nil
		index: 0
		parentNode: nil!

referencesFromNode: aNode 
	" If the node aNode can be expanded, the receiver expands it."

	aNode canExpand ifTrue: [self expandNode: aNode].
	^aNode! !

!GSPTreeNodeVisitor methodsFor: 'initialize-release'!

initialize
	self treeNodeBuilder: self treeNodeBuilderClass new! !

!GSPTreeNodeVisitor methodsFor: 'private'!

expandNode: aNode 
	" The receiver expands aNode, in collaboration with its builder of nodes."

	aNode expandWithTreeNodeBuilder: self treeNodeBuilder!

treeNodeBuilder
	" The receiver returns the object qualified to create nodes."

	^treeNodeBuilder!

treeNodeBuilder: anObject 
	" The receiver stores the object qualified to create nodes."

	treeNodeBuilder := anObject!

treeNodeBuilderClass
	" The receiver returns an appropriate class for the object that to create nodes."

	^GSPTreeNodeBuilder! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPTreeNodeVisitor class
	instanceVariableNames: ''!


!GSPTreeNodeVisitor class methodsFor: 'instance-creation'!

new
	^super new initialize!

referencesFromNode: anObject 
	^self new referencesFromNode: anObject! !

