Object subclass: #GSPTreeNode
	instanceVariableNames: 'value name index parentNode forbiddenSubstitutions '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPTreeNode comment:
'I am an abstract class. I represent the nodes of the reference tree used to inspect objects. I know a part of the object that begins to be inspected. This part can be the value of an instance variable or an element inside a collection. In the first case, I know the name of the instance variable that contains it and the index of the same one. In the second case, I know the key inside the collection where the stored value is. Also, I know which variables of instance of the object that I represent cannot be modified, and who is my node father inside the reference tree.

Subclasses must implement the following messages:
	testing
		canExpand

Instance Variables:
	forbiddenSubstitutions	<Collection>	description of forbiddenSubstitutions
	index	<Object>	description of index
	name	<ExceptionSet | GenericException class | SequenceableCollection | Signal>	description of name
	parentNode	<TreeNode>	description of parentNode
	value	<>	description of value

'!


!GSPTreeNode methodsFor: 'private'!

evaluate: aNewValue 
	"The receiver returns the result of evaluating the contained text in aNewValue."

	^Inspector new 
		evaluateText: aNewValue
		from: nil
		ifFail: [^self errorSignal]!

forbiddenSubstitutions
	"The receiver returns the collection of names of variables whose values cannot be altered."

	^forbiddenSubstitutions!

forbiddenSubstitutions: anObject 
	"The receiver stores the collection of names of variables whose values cannot be altered."

	forbiddenSubstitutions := anObject!

newNodeBuiltBy: aTNBuilder with: aValue 
	"The receiver makes and returns a new node with the value of aValue. "

	^aTNBuilder 
		createNodeForObject: aValue
		name: self name
		index: self index
		parentNode: self parentNode!

primitiveReplaceNode: oldNode withNewNode: newNode 
	"The receiver replaces a node son (oldNode) for one new (newNode). This message should be redefined by the subclasses. "!

replaceNode: oldNode withNewNode: newNode 
	"If the node oldNode represents a variable that can be replaced, it makes the substitution."

	(self forbiddenSubstitutions includes: oldNode name)
		ifFalse: [self primitiveReplaceNode: oldNode withNewNode: newNode]! !

!GSPTreeNode methodsFor: 'accessing'!

acceptNewValue: aNewValue with: aTNBuilder 
	" The receiver receives a new value (aNewValue) to replace its current value. It notifies to their  father node that the value of the variable that represents has changed. "

	| result newNode |
	result := self evaluate: aNewValue.
	newNode := self newNodeBuiltBy: aTNBuilder with: result.
	self parentNode replaceNode: self withNewNode: newNode!

acceptStringAsValue: aText with: aTNBuilder 
	" The receiver receives a new string as value in a Text, to replace its current value. It notifies to their father node that the value of the variable that represents has changed. "

	| newNode |
	newNode := self newNodeBuiltBy: aTNBuilder with: aText string.
	self parentNode replaceNode: self withNewNode: newNode!

expandWithTreeNodeBuilder: aTNBuilder 
	" The receiver expands creating children (nodes) that represent its variables. This message should be redefined by those subclasses whose instances have instance variables. "!

index
	" The receiver returns the index of instance variable that has inside its object father (that that is represented by parentNode). "

	^index!

index: anObject 
	" The receiver stores the index of instance variable that has inside its object father (that that is represented by parentNode). "

	index := anObject!

name
	" The receiver returns the name of instance variable that represents inside its object father (that that is represented by parentNode). "

	^name!

name: anObject 
	" The receiver stores the name of instance variable that represents inside its object father (that that is represented by parentNode). "

	name := anObject!

parentNode
	" The receiver returns its node father. "

	^parentNode!

parentNode: anObject 
	" The receiver stores its node father. "

	parentNode := anObject!

value
	" The receiver returns the value of the instance variable that represents. "

	^value!

value: anObject 
	" The receiver stores the value of the instance variable that represents. "

	value := anObject!

valueImage
	" The receiver returns a graphic representation of the type of the value of the instance variable that represents. "

	^self class icon! !

!GSPTreeNode methodsFor: 'printing'!

displayString
	" The receiver returns a more complete identification that a simple printString. "

	| strValue |
	strValue := self value printString.
	self name isNil ifTrue: [^strValue].
	^self name , ' [' , strValue , ']'!

displayValue
	" The receiver returns a string that represents its value. "

	| str |
	str := ReadWriteStream on: ''.
	self value printOn: str.
	^str contents! !

!GSPTreeNode methodsFor: 'initialize-release'!

initialize
	" The receiver initializes the variables that  need to be initialized. "

	self initializeChildren.
	self initializeForbiddenSubstitutions!

initializeChildren
	" The receiver is initialized without any son. "

	self children: nil.
	self expanded: false!

initializeForbiddenSubstitutions
	" The receiver initializes the collection of names of the variables of instances (whose value contains them) that cannot be modified. This message should be redefined by those subclasses whose instances cannot modify all the variables of instances of its value. "

	self forbiddenSubstitutions: #()!

initializeWithObject: anObject name: aName index: anIndex parentNode: aParent 
	" The receiver is initialized with the values that pass him as parameter. "

	self value: anObject.
	self name: aName.
	self index: anIndex.
	self parentNode: aParent.
	self initialize! !

!GSPTreeNode methodsFor: 'testing'!

canChange
	" The receiver returns true if it can be replaced, otherwise it returns false. "

	^self parentNode isNil not!

canCompile
	" The receiver returns true if its value can be compiled, otherwise it returns false. This message should be redefined by those subclasses whose instances can compile its value. For defect this message returns false . "

	^false!

canExpand
	" The receiver returns true if it can expand, false otherwise. The subclasses should redefine this message. "

	^self subclassResponsibility! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPTreeNode class
	instanceVariableNames: ''!


!GSPTreeNode class methodsFor: 'instance creation'!

newWithObject: anObject name: aName index: anIndex parentNode: aParent 
	^super new 
		initializeWithObject: anObject
		name: aName
		index: anIndex
		parentNode: aParent! !

GSPTreeNode subclass: #GSPSingleTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPSingleTreeNode comment:
'I represent the nodes whose values don''t have instance variables, they are not collections and they are not magnitudes.

'!


!GSPSingleTreeNode methodsFor: 'testing'!

canExpand
	" The receiver returns false because it cannot be expanded."

	^false! !

!GSPSingleTreeNode methodsFor: 'accessing'!

children
	" The receiver returns an empty collection because it doesn't have children."

	^#()!

children: aCollection 
	" The receiver receives a collection that doesn't store because it cannot contain children. "!

expanded
	" The receiver returns true since, for not having variables of instances, it is expanded in its entirety. "

	^true!

expanded: aValue 
	" The receiver receives a boolean that defines its expansion state, because it is always expanded, it doesn't store it. "! !

GSPTreeNode subclass: #GSPComposedTreeNode
	instanceVariableNames: 'children expanded '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPComposedTreeNode comment:
'I represent the nodes whose value has either  instance variables or it is a collection. I have a collection of nodes that represent the values of this variables or the values of a collection. My instances can be expanded (when they know all their nodes children) or without expanding (when they have not still expanded).

Instance Variables:
	children	<SequenceableCollection>	description of children
	expanded	<AbstractCondition | Boolean | BrowserEnvironment>	description of expanded

'!


!GSPComposedTreeNode methodsFor: 'private'!

changeInstVarValueAt: anIndex withOldValue: oldValue withNewValue: newValue 
	" The receiver replaces the old value (oldValue) that is in the instance variable with index anIndex with the new value (newValue). This message should be redefined by those subclasses whose instances contain collections as value ."

	self value instVarAt: anIndex put: newValue!

primitiveReplaceNode: oldNode withNewNode: newNode 
	"The receiver replaces a node son (oldNode) for one new (newNode)."

	self 
		changeInstVarValueAt: oldNode index
		withOldValue: oldNode value
		withNewValue: newNode value.
	self children replaceAll: oldNode with: newNode!

variableNamesAndValue
	" The receiver collects and returns a dictionary of the instance variables of its value with the 	following 	data:  
			As key: the index of instance variable  
     		As value: an association between the name of the instance variable and their value."

	| varNamesAndValue variables |
	varNamesAndValue := Dictionary new.
	variables := self value class allInstVarNames.
	1 to: variables size
		do: 
			[:i | 
			varNamesAndValue at: i put: (variables at: i) -> (self value instVarAt: i)].
	^varNamesAndValue! !

!GSPComposedTreeNode methodsFor: 'accessing'!

children
	" The receiver returns its children."

	^children isNil 
		ifTrue: [children := OrderedCollection new]
		ifFalse: [children]!

children: aCollection 
	" The receiver stores its children."

	children := aCollection!

expanded
	" The receiver returns true if it contains to its nodes children, false otherwise."

	^expanded!

expanded: anObject 
	" The receiver stores true if it is expanded, false otherwise."

	expanded := anObject!

expandWithTreeNodeBuilder: aTNBuilder 
	"The receiver expands creating children (nodes) that represent its variables."

	self variableNamesAndValue
		associationsDo: 
			[:indexNameValue | 
			| child |
			child := aTNBuilder
						createNodeForObject: indexNameValue value value
						name: indexNameValue value key
						index: indexNameValue key
						parentNode: self.
			self addChild: child].
	self expanded: true! !

!GSPComposedTreeNode methodsFor: 'testing'!

canExpand
	" The receiver returns true if it can be expanded, in another case returns false. "

	^self expanded not! !

!GSPComposedTreeNode methodsFor: 'adding'!

addChild: aChild 
	" The receiver adds a son. "

	self children add: aChild! !

GSPComposedTreeNode subclass: #GSPDefaultTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPDefaultTreeNode comment:
'My instances contain as value an instance of some subclase of Object.
'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPDefaultTreeNode class
	instanceVariableNames: ''!


!GSPDefaultTreeNode class methodsFor: 'resources'!

icon
	"UIMaskEditor new openOnClass: self andSelector: #default"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue red; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 4227 scaledGreen: 0 scaledBlue: 0); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 64 5 88 37 10 129 88 37 41 97 88 37 41 97 88 37 41 97 88 37 41 97 88 37 10 129 88 37 64 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

GSPComposedTreeNode subclass: #GSPBehaviorTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPBehaviorTreeNode comment:
'My instances contain as value some subclass of Behavior.
'!


!GSPBehaviorTreeNode methodsFor: 'printing'!

displayString
	" The receiver returns a more complete identification that a simple printString. "

	self name isNil ifTrue: [^'[' , self value printString , ']'].
	^self name , ' [' , self value printString , ']'! !

!GSPBehaviorTreeNode methodsFor: 'initialize-release'!

initializeForbiddenSubstitutions
	" The receiver initializes the collection of names of the variables of instances (whose value contains them) that cannot be modified. The nodes that represent behavior subclasses cannot modify their name."

	self forbiddenSubstitutions: #('name')! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPBehaviorTreeNode class
	instanceVariableNames: ''!


!GSPBehaviorTreeNode class methodsFor: 'resources'!

icon
	"UIMaskEditor new openOnClass: self andSelector: #behavior"

	<resource: #image>
	^CachedImage on: (Image 
				extent: 16 @ 16
				depth: 2
				bitsPerPixel: 2
				palette: (MappedPalette withColors: ((Array new: 3)
								at: 1 put: ColorValue magenta;
								at: 2 put: ColorValue white;
								at: 3
									put: (ColorValue 
											scaledRed: 4227
											scaledGreen: 0
											scaledBlue: 4227);
								yourself))
				usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 0 5 88 37 42 129 88 37 41 97 88 37 42 133 88 37 41 97 88 37 41 97 88 37 42 129 88 37 0 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

GSPComposedTreeNode subclass: #GSPBlockClosureTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPBlockClosureTreeNode comment:
'My instances contain as value some instance of BlockClosure.

'!


!GSPBlockClosureTreeNode methodsFor: 'private'!

variableNamesAndValue
	" The receiver collects and returns a dictionary of the instance variables of its value with the 	following 	data:  
			As key: the index of instance variable  
     		As value: an association between the name of the instance variable and their value.
	Also, it adds the source code and the bytes codes of the method of their value."

	| varNamesAndValue |
	varNamesAndValue := super variableNamesAndValue.
	varNamesAndValue at: varNamesAndValue size + 1
		put: ' - source' -> self value method getSource.
	varNamesAndValue at: varNamesAndValue size + 1
		put: ' - bytes code' -> self value method symbolic.
	^varNamesAndValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPBlockClosureTreeNode class
	instanceVariableNames: ''!


!GSPBlockClosureTreeNode class methodsFor: 'resources'!

icon
	"UIMaskEditor new openOnClass: self andSelector: #code"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue red; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 4227 scaledGreen: 0 scaledBlue: 0); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 64 5 88 37 10 129 88 37 41 97 88 37 41 97 88 37 41 97 88 37 41 97 88 37 10 129 88 37 64 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

GSPSingleTreeNode subclass: #GSPNullTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPNullTreeNode comment:
'My instances contain nil as value.
'!


!GSPNullTreeNode methodsFor: 'printing'!

displayString
	" The receiver returns a more complete identification that a simple printString. "

	| strValue |
	strValue := ' [' , self value printString , ']'.
	self name isNil ifTrue: [^strValue].
	^self name , strValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPNullTreeNode class
	instanceVariableNames: ''!


!GSPNullTreeNode class methodsFor: 'resources'!

icon
	"UIMaskEditor new openOnClass: self andSelector: #null"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue cyan; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 0 scaledGreen: 4227 scaledBlue: 4227); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 0 65 88 37 40 33 88 37 42 33 88 37 40 161 88 37 40 97 88 37 40 97 88 37 40 97 88 37 0 65 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

GSPComposedTreeNode subclass: #GSPCompiledCodeTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPCompiledCodeTreeNode comment:
'My instances contain as value an instance of some subclass of CompiledCode.
'!


!GSPCompiledCodeTreeNode methodsFor: 'printing'!

displayValue
	" The receiver returns a string that represents its value. "

	| str |
	str := ReadWriteStream on: ''.
	str nextPutAll: self value getSource.
	^str contents! !

!GSPCompiledCodeTreeNode methodsFor: 'private'!

variableNamesAndValue
	" The receiver collects and returns a dictionary of the instance variables of its value with the 	following 	data:  
			As key: the index of instance variable  
     		As value: an association between the name of the instance variable and their value.
	Also, it adds the source code and the bytes codes of their value."

	| varNamesAndValue |
	varNamesAndValue := super variableNamesAndValue.
	varNamesAndValue at: varNamesAndValue size + 1
		put: ' - source' -> self value getSource.
	varNamesAndValue at: varNamesAndValue size + 1
		put: ' - bytes code' -> self value symbolic.
	^varNamesAndValue! !

!GSPCompiledCodeTreeNode methodsFor: 'accessing'!

acceptNewValue: aText with: aTNBuilder 
	"The receiver receives a new code for your value (a compiled code) to replace the current code. It 
	notifies to their father node that the code has changed."

	| selector oldCode heading |
	oldCode := self value getSource.
	heading := self value mclass whichCategoryIncludesSelector: (self value mclass selectorAtMethod: self value ifAbsent: [^self errorSignal]).
	[selector := self value mclass compile: aText string classified: heading]
		on: Error
		do: 
			[:exc | 
			self value mclass compile: oldCode classified: heading.
			^self errorSignal].
	self value: (self value mclass compiledMethodAt: selector).
	self index: selector.
	self parentNode replaceNode: self withNewNode: self! !

!GSPCompiledCodeTreeNode methodsFor: 'testing'!

canCompile
	" The receiver true returns because its value can be compiled."

	^true! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPCompiledCodeTreeNode class
	instanceVariableNames: ''!


!GSPCompiledCodeTreeNode class methodsFor: 'resources'!

icon
	"UIMaskEditor new openOnClass: self andSelector: #code"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue red; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 4227 scaledGreen: 0 scaledBlue: 0); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 64 5 88 37 10 129 88 37 41 97 88 37 41 97 88 37 41 97 88 37 41 97 88 37 10 129 88 37 64 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

GSPComposedTreeNode subclass: #GSPContextTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPContextTreeNode comment:
'My instances contain as value an instance of some subclass of Context.
'!


!GSPContextTreeNode methodsFor: 'printing'!

displayValue
	" The receiver returns a string that represents its value. "

	| str |
	str := ReadWriteStream on: ''.
	str nextPutAll: self value sourceCode.
	^str contents! !

!GSPContextTreeNode methodsFor: 'private'!

primitiveReplaceNode: oldNode withNewNode: newNode 
	"The receiver replaces a node son (oldNode) for one new (newNode). In case the variable to modify of its value is the method, the value is resized with the new method."

	oldNode name = 'method' 
		ifTrue: 
			[self value: (self value resizedWith: newNode value).
			self children replaceAll: oldNode with: newNode]
		ifFalse: [super primitiveReplaceNode: oldNode withNewNode: newNode]!

variableNamesAndValue
	" The receiver collects and returns a dictionary of the instance variables of its value with the 	following 	data:  
			As key: the index of instance variable  
     		As value: an association between the name of the instance variable and their value.
	Also, it adds the source code and the bytes codes of the method of their value."

	| varNamesAndValue |
	varNamesAndValue := super variableNamesAndValue.
	varNamesAndValue at: varNamesAndValue size + 1
		put: ' - source' -> self value method getSource.
	varNamesAndValue at: varNamesAndValue size + 1
		put: ' - bytes code' -> self value method symbolic.
	^varNamesAndValue! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPContextTreeNode class
	instanceVariableNames: ''!


!GSPContextTreeNode class methodsFor: 'resources'!

icon
	"UIMaskEditor new openOnClass: self andSelector: #code"

	<resource: #image>
	^CachedImage on: (Image extent: 16@16 depth: 2 bitsPerPixel: 2 palette: (MappedPalette withColors: ((Array new: 3) at: 1 put: ColorValue red; at: 2 put: ColorValue white; at: 3 put: (ColorValue scaledRed: 4227 scaledGreen: 0 scaledBlue: 0); yourself)) usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 64 5 88 37 10 129 88 37 41 97 88 37 41 97 88 37 41 97 88 37 41 97 88 37 10 129 88 37 64 5 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

GSPComposedTreeNode subclass: #GSPMagnitudeTreeNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GSP- Inspector TreeNodes'!
GSPMagnitudeTreeNode comment:
'My instances contain as value an instance of some subclase of Magnitud.
'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GSPMagnitudeTreeNode class
	instanceVariableNames: ''!


!GSPMagnitudeTreeNode class methodsFor: 'resources'!

icon
	"UIMaskEditor new openOnClass: self andSelector: #magnitude"

	<resource: #image>
	^CachedImage on: (Image 
				extent: 16 @ 16
				depth: 2
				bitsPerPixel: 2
				palette: (MappedPalette withColors: ((Array new: 3)
								at: 1 put: ColorValue yellow;
								at: 2 put: ColorValue white;
								at: 3
									put: (ColorValue 
											scaledRed: 4227
											scaledGreen: 4227
											scaledBlue: 0);
								yourself))
				usingBits: #[0 0 0 0 42 170 170 168 37 85 85 88 37 85 85 88 37 1 65 88 37 33 33 88 37 40 161 88 37 34 33 88 37 32 33 88 37 33 33 88 37 33 33 88 37 1 1 88 37 85 85 88 37 85 85 88 42 170 170 168 0 0 0 0])! !

